package com.fr.plugin.cpic.db.dao;

import com.fanruan.api.util.StringKit;
import com.fr.plugin.cpic.db.entity.CpicEntryEntity;
import com.fr.stable.db.dao.BaseDAO;
import com.fr.stable.db.dao.DAOProvider;
import com.fr.stable.db.session.DAOSession;
import com.fr.stable.query.QueryFactory;
import com.fr.stable.query.condition.QueryCondition;
import com.fr.stable.query.restriction.RestrictionFactory;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class CpicEntryDao extends BaseDAO<CpicEntryEntity> {

    public CpicEntryDao(DAOSession daoSession) {
        super(daoSession);
    }

    /**
     * 如果重写了getEntityClass方法，就可以不重写其他所有DAO<T>接口中的方法
     * 推荐只重写getEntityClass方法，保留原写法作为兼容
     *
     * @return
     */
    @Override
    protected Class<CpicEntryEntity> getEntityClass() {
        return CpicEntryEntity.class;
    }

    public final static DAOProvider DAO = new DAOProvider() {
        @Override
        public Class getEntityClass() {
            return CpicEntryEntity.class;
        }

        @Override
        public Class<? extends BaseDAO> getDAOClass() {
            return CpicEntryDao.class;
        }
    };

    public CpicEntryEntity findById(String id) throws Exception {
        QueryCondition condition = QueryFactory.create();
        if (StringKit.isNotBlank(id)) {
            condition.addRestriction(RestrictionFactory.eq("id", id));
            condition.addRestriction(RestrictionFactory.eq("del", "0"));
        } else {
            return null;
        }
        return findOne(condition);
    }

    public List<CpicEntryEntity> findEntryList(String homeId) throws Exception {
        QueryCondition condition = QueryFactory.create();
        if (StringKit.isNotBlank(homeId)) {
            condition.addRestriction(RestrictionFactory.eq("homeId", homeId));
            condition.addRestriction(RestrictionFactory.eq("del", "0"));
        } else {
            return null;
        }
        return find(condition);
    }

    public void realDel(String id) throws Exception {
        QueryCondition condition = QueryFactory.create();
        if (StringKit.isNotBlank(id)) {
            condition.addRestriction(RestrictionFactory.eq("id", id));
            remove(condition);
        }
    }

    public void logicDelByHomeId(String homeId) throws Exception {
        Map<String, Object> updateParams = new HashMap<>();
        updateParams.put("del", "1");
        QueryCondition condition = QueryFactory.create();
        if (StringKit.isNotBlank(homeId)) {
            condition.addRestriction(RestrictionFactory.eq("homeId", homeId));
            update(updateParams, condition);
        }
    }
}
