package com.fr.plugin.cpic.db.service;

import com.fanruan.api.log.LogKit;
import com.fanruan.api.util.StringKit;
import com.fr.plugin.cpic.db.CpicDBAccessProvider;
import com.fr.plugin.cpic.db.bean.CpicUserBean;
import com.fr.plugin.cpic.db.dao.CpicEntryDao;
import com.fr.plugin.cpic.db.dao.CpicUserDao;
import com.fr.plugin.cpic.db.entity.CpicUserEntity;
import com.fr.stable.core.UUID;
import com.fr.stable.db.action.DBAction;
import com.fr.stable.db.dao.DAOContext;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

public class CpicUserService {

    /**
     * 新增
     */
    public static void saveOrUpdate(CpicUserBean bean) {
        try {
            CpicDBAccessProvider.getAccessor().runDMLAction(new DBAction<Boolean>() {
                @Override
                public Boolean run(DAOContext context) throws Exception {
                    CpicUserEntity targetEntity = null;
                    // 尝试查询
                    if (bean.getId() != null && StringKit.isNotBlank(bean.getId())) {
                        targetEntity = context.getDAO(CpicUserDao.class).findById(bean.getId());
                    }

                    if (targetEntity != null) {
                        // 更新
                        targetEntity.setHomeId(bean.getHomeId());
                        targetEntity.setHomeName(bean.getHomeName());
                        targetEntity.setUserId(bean.getUserId());
                        targetEntity.setUserName(bean.getUserName());
                        targetEntity.setUserAccount(bean.getUserAccount());
                        context.getDAO(CpicUserDao.class).update(targetEntity);
                    } else {
                        // 新增
                        targetEntity = new CpicUserEntity();
                        targetEntity.setId(UUID.randomUUID().toString().toLowerCase());
                        targetEntity.setHomeId(bean.getHomeId());
                        targetEntity.setHomeName(bean.getHomeName());
                        targetEntity.setUserId(bean.getUserId());
                        targetEntity.setUserName(bean.getUserName());
                        targetEntity.setUserAccount(bean.getUserAccount());
                        targetEntity.setDel("0");
                        context.getDAO(CpicUserDao.class).add(targetEntity);
                    }
                    return true;
                }
            });
        } catch (Exception e) {
            LogKit.error("新增/更新失败:{}", e.getMessage());
        }
    }

    /**
     * 查询用户
     */
    public static CpicUserBean findHomeUserByUserId(String homeId, String userId) {
        try {
            return CpicDBAccessProvider.getAccessor().runQueryAction(new DBAction<CpicUserBean>() {
                @Override
                public CpicUserBean run(DAOContext context) throws Exception {
                    CpicUserEntity item = context.getDAO(CpicUserDao.class).findHomeUserByUserId(homeId, userId);
                    return entityToBean(item);
                }
            });
        } catch (Throwable e) {
            LogKit.error("查询失败:{}", e.getMessage());
        }
        return null;
    }

    /**
     * 查询列表
     */
    public static List<CpicUserBean> findList(String homeId) {
        try {
            return CpicDBAccessProvider.getAccessor().runQueryAction(new DBAction<List<CpicUserBean>>() {
                @Override
                public List<CpicUserBean> run(DAOContext context) throws Exception {
                    List<CpicUserEntity> items = context.getDAO(CpicUserDao.class).findUserList(homeId);
                    List<CpicUserBean> result = new ArrayList<>();
                    for (CpicUserEntity item : items) {
                        result.add(entityToBean(item));
                    }
                    return result;
                }
            });
        } catch (Throwable e) {
            LogKit.error("查询失败:{}", e.getMessage());
        }
        return Collections.emptyList();
    }

    /**
     * 删除
     *
     * @return
     */
    public static void realDel(String id) {
        try {
            CpicDBAccessProvider.getAccessor().runDMLAction(new DBAction<Boolean>() {
                @Override
                public Boolean run(DAOContext context) throws Exception {
                    context.getDAO(CpicUserDao.class).realDel(id);
                    return true;
                }
            });
        } catch (Exception e) {
            LogKit.error("删除失败:{}", e.getMessage());
        }
    }

    public static CpicUserBean entityToBean(CpicUserEntity entity) {
        if (null == entity) {
            return null;
        }
        CpicUserBean bean = new CpicUserBean();
        bean.setId(entity.getId());
        bean.setHomeId(entity.getHomeId());
        bean.setHomeName(entity.getHomeName());
        bean.setUserId(entity.getUserId());
        bean.setUserName(entity.getUserName());
        bean.setUserAccount(entity.getUserAccount());
        return bean;
    }

}
