package com.fr.plugin.cpic.utils;

import com.fanruan.api.json.JSONKit;
import com.fanruan.api.log.LogKit;
import com.fanruan.api.util.StringKit;
import com.fr.decision.authority.data.CustomRole;
import com.fr.decision.authority.data.Department;
import com.fr.decision.webservice.bean.entry.EntryBean;
import com.fr.json.JSONArray;
import com.fr.plugin.cpic.db.bean.CpicEntryBean;
import com.fr.plugin.cpic.web.bean.TreeItemBean;
import com.fr.plugin.cpic.web.bean.UserMenuBean;
import com.fr.third.org.apache.commons.collections4.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

public class ConvertUtil {

    public static List<TreeItemBean> toTree(List<EntryBean> entryBeanList, List<CpicEntryBean> cpicEntryBeanList) {
        // 转换
        List<TreeItemBean> items = ConvertUtil.toTreeItems(entryBeanList, cpicEntryBeanList);
        // 生成树
        List<TreeItemBean> tree = ConvertUtil.toTreeItemBeanList(items);
        return tree;
    }

    /**
     * 目录列表转换为List<TreeItemBean>
     *
     * @param entryBeanList
     * @return
     */
    public static List<TreeItemBean> toTreeItems(List<EntryBean> entryBeanList, List<CpicEntryBean> cpicEntryBeanList) {
        List<TreeItemBean> list = new ArrayList<>();
        for (EntryBean entryBean : entryBeanList) {
            TreeItemBean entry = ConvertUtil.toTreeItem(entryBean, cpicEntryBeanList);
            if (null != entry) {
                list.add(entry);
            }
        }
        return list;
    }

    /**
     * 转换为TreeItemBean，同时设置是否选中
     *
     * @param entryBean
     * @param cpicEntryBeanList
     * @return
     */
    public static TreeItemBean toTreeItem(EntryBean entryBean, List<CpicEntryBean> cpicEntryBeanList) {
        TreeItemBean item = null;
        if (entryBean != null) {
            item = new TreeItemBean();
            // 是否根目录
            if ("decision-directory-root".equals(entryBean.getId())) {
                item.setTitle("根目录");
                item.setId(entryBean.getId());
                item.setPid(StringKit.isBlank(entryBean.getpId()) ? "" : entryBean.getpId());
                item.setField("");
                item.setHref("");
                item.setSpread(true);
                item.setChecked(false);
                item.setDisabled(true);
            } else {
                // 查询是否存在配置
//                CpicEntryBean cpicEntryBean = cpicEntryBeanList.stream().filter(bean -> bean.getEntryId().equals(entryBean.getId())).findFirst().orElse(null);
                item.setTitle(entryBean.getText());
                item.setId(entryBean.getId());
                item.setPid(StringKit.isBlank(entryBean.getpId()) ? "" : entryBean.getpId());
                item.setField("");
                item.setHref("");
                item.setSpread(false);
                item.setChecked(false);
//                if (null == cpicEntryBean) {
//                } else {
//                    item.setChecked(true);
//                }
                item.setDisabled(false);
            }
        }
        return item;
    }

    // 返回构建好的树列表
    public static List<TreeItemBean> toTreeItemBeanList(List<TreeItemBean> allTreeItemBeanList) {
        // 获取所有根节点
        List<TreeItemBean> treeList = new ArrayList<>();
        List<String> rootCode = new ArrayList<>();
        for (TreeItemBean treeItemBean : allTreeItemBeanList) {
            if (rootCode.contains(treeItemBean.getId())) {
                continue;
            }
            // 根据该节点查询根节点
            TreeItemBean parent = ConvertUtil.getParentCode(allTreeItemBeanList, treeItemBean);
            if (rootCode.contains(parent.getId())) {
                continue;
            }
            treeList.add(parent);
            rootCode.add(parent.getId());
        }

        // 根据所有根节点分别构建树
        for (TreeItemBean parentItem : treeList) {
            ConvertUtil.buildTree(allTreeItemBeanList, parentItem);
        }

        // 输出一下结果
        JSONArray jsonArray = JSONKit.createJSONArray(treeList);
        LogKit.debug(jsonArray.toString());

        return treeList;
    }

    // 构建树
    private static void buildTree(List<TreeItemBean> allTreeItemBeanList, TreeItemBean parentTreeItemBean) {
        // 查询下级节点
        List<TreeItemBean> childs = allTreeItemBeanList.stream().filter(item -> item.getPid().equals(parentTreeItemBean.getId())).collect(Collectors.toList());
        // 下级节点继续构建
        if (CollectionUtils.isNotEmpty(childs)) {
            for (TreeItemBean treeItemBean : childs) {
                ConvertUtil.buildTree(allTreeItemBeanList, treeItemBean);
            }
        }
        parentTreeItemBean.setChildren(childs);
    }

    // 返回根节点
    private static TreeItemBean getParentCode(List<TreeItemBean> allTreeItemBeanList, TreeItemBean treeItemBean) {
        boolean haveNext = true;
        List<TreeItemBean> parentTreeItemBeans;
        TreeItemBean parentTreeItemBean = treeItemBean;
        while (haveNext) {
            TreeItemBean finalParentTreeItemBean = parentTreeItemBean;
            parentTreeItemBeans = allTreeItemBeanList.stream().filter(item -> item.getId().equals(finalParentTreeItemBean.getPid())).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(parentTreeItemBeans)) {
                // 有父节点
                parentTreeItemBean = parentTreeItemBeans.get(0);
            } else {
                haveNext = false;
            }

        }
        return parentTreeItemBean;
    }

    /**
     * 转换list
     *
     * @param tree
     * @return
     */
    public static List<CpicEntryBean> itemsToEntrys(List<TreeItemBean> tree) {
        List<CpicEntryBean> result = new ArrayList<>();
        CpicEntryBean entryBean;
        for (TreeItemBean item : tree) {
            if ("decision-directory-root".equals(item.getId())) {
                continue;
            }
            entryBean = new CpicEntryBean();
            entryBean.setEntryId(item.getId());
            entryBean.setEntryName(item.getTitle());
            result.add(entryBean);
        }
        return result;
    }

    /**
     * tree转换为list
     *
     * @param tree
     * @return
     */
    public static List<CpicEntryBean> treeToList(List<TreeItemBean> tree) {
        List<CpicEntryBean> result = new ArrayList<>();
        for (TreeItemBean item : tree) {
            ConvertUtil.buildList(result, item);
        }
        return result;
    }

    public static void buildList(List<CpicEntryBean> result, TreeItemBean treeBean) {
        List<TreeItemBean> childrens = treeBean.getChildren();
        if (CollectionUtils.isNotEmpty(childrens)) {
            CpicEntryBean entryBean;
            for (TreeItemBean item : childrens) {
                ConvertUtil.buildList(result, item);
                entryBean = new CpicEntryBean();
                entryBean.setEntryId(item.getId());
                entryBean.setEntryName(item.getTitle());
                result.add(entryBean);
            }
        }
    }

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////

    public static List<UserMenuBean> deptListToMenuList(List<Department> departmentList) {
        // 转换
        List<UserMenuBean> list = new ArrayList<>();
        UserMenuBean userMenuBean;
        for (Department department : departmentList) {
            userMenuBean = new UserMenuBean();
            userMenuBean.setId(department.getId());
            userMenuBean.setTitle(department.getName());
            userMenuBean.setMtype("department");
            userMenuBean.setPid(department.getParentId());
            list.add(userMenuBean);
        }
        return list;
    }
    public static List<UserMenuBean> roleListToMenuList(List<CustomRole> departmentList) {
        // 转换
        List<UserMenuBean> list = new ArrayList<>();
        UserMenuBean userMenuBean;
        for (CustomRole department : departmentList) {
            userMenuBean = new UserMenuBean();
            userMenuBean.setId(department.getId());
            userMenuBean.setTitle(department.getName());
            userMenuBean.setMtype("customrole");
//            userMenuBean.setPid(department.getParentId());
            list.add(userMenuBean);
        }
        return list;
    }

    public static List<UserMenuBean> departmentListToTree(List<Department> departmentList) {
        // 转换
        List<UserMenuBean> list = ConvertUtil.deptListToMenuList(departmentList);
        // 生成树
        List<UserMenuBean> tree = ConvertUtil.departmentListToTreeItemBeanList(list);
        return tree;
    }

    private static List<UserMenuBean> departmentListToTreeItemBeanList(List<UserMenuBean> userMenuBeanList) {
        // 获取所有根节点
        List<UserMenuBean> treeList = new ArrayList<>();
        List<String> rootCode = new ArrayList<>();
        for (UserMenuBean treeItemBean : userMenuBeanList) {
            if (rootCode.contains(treeItemBean.getId())) {
                continue;
            }
            // 根据该节点查询根节点
            UserMenuBean parent = ConvertUtil.getParentCode2(userMenuBeanList, treeItemBean);
            if (rootCode.contains(parent.getId())) {
                continue;
            }
            treeList.add(parent);
            rootCode.add(parent.getId());
        }

        // 根据所有根节点分别构建树
        for (UserMenuBean parentItem : treeList) {
            ConvertUtil.buildTree2(userMenuBeanList, parentItem);
        }
        return treeList;
    }

    // 返回根节点
    private static UserMenuBean getParentCode2(List<UserMenuBean> allTreeItemBeanList, UserMenuBean treeItemBean) {
        boolean haveNext = true;
        List<UserMenuBean> parentTreeItemBeans;
        UserMenuBean parentTreeItemBean = treeItemBean;
        while (haveNext) {
            UserMenuBean finalParentTreeItemBean = parentTreeItemBean;
            parentTreeItemBeans = allTreeItemBeanList.stream().filter(item -> item.getId().equals(finalParentTreeItemBean.getPid())).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(parentTreeItemBeans)) {
                // 有父节点
                parentTreeItemBean = parentTreeItemBeans.get(0);
            } else {
                haveNext = false;
            }

        }
        return parentTreeItemBean;
    }

    // 构建树
    private static void buildTree2(List<UserMenuBean> allTreeItemBeanList, UserMenuBean parentTreeItemBean) {
        // 查询下级节点
        List<UserMenuBean> childs = allTreeItemBeanList.stream().filter(item -> item.getPid().equals(parentTreeItemBean.getId())).collect(Collectors.toList());
        // 下级节点继续构建
        if (CollectionUtils.isNotEmpty(childs)) {
            for (UserMenuBean treeItemBean : childs) {
                ConvertUtil.buildTree2(allTreeItemBeanList, treeItemBean);
            }
        }
        parentTreeItemBean.setChildren(childs);
    }
}
