package com.fr.plugin.cpic.utils;

import com.fr.stable.StringUtils;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

public class UrlUtils {

    /**
     * 获取URL的参数列表
     *
     * @param url
     * @return
     */
    public static String getBaseUrl(String url) {
        String baseUrl = "";
        if (StringUtils.isBlank(url)) {
            return baseUrl;
        }
        url = url.trim();
        String[] urlParts = url.split("\\?");
        //没有参数
        if (urlParts.length >= 1) {
            baseUrl = urlParts[0];
        }
        return baseUrl;
    }

    /**
     * 获取URL的参数列表
     *
     * @param url
     * @return
     */
    public static Map<String, String> getParams(String url) {
        Map<String, String> params = new HashMap<>();
        if (StringUtils.isBlank(url)) {
            return params;
        }
        url = url.trim();
        String[] urlParts = url.split("\\?");
        //没有参数
        if (urlParts.length <= 1) {
            return params;
        }
        //有参数
        String[] urlParams = urlParts[1].split("&");
        for (String param : urlParams) {
            String[] keyValue = param.split("=");
            if (keyValue.length < 2) {
                continue;
            }
            params.put(keyValue[0], keyValue[1]);
        }
        return params;
    }

    /**
     * 生成完整url
     *
     * @param baseUrl
     * @param params
     * @return
     */
    public static String getUrl(String baseUrl, Map<String, String> params) {
        StringBuilder sb = new StringBuilder();
        sb.append(baseUrl);
        if (null != params && params.size() > 0) {
            sb.append("?");
            for (Entry<String, String> entry : params.entrySet()) {
                sb.append(entry.getKey()).append("=").append(entry.getValue()).append("&");
            }
        }
        String url = sb.toString();
        if (url.endsWith("&")) {
            url = url.substring(0, url.length() - 1);
        }
        return url;
    }

}
