package com.fr.plugin.cpic.web.custom;

import com.fanruan.api.util.StringKit;
import com.fr.base.ServerConfig;
import com.fr.base.email.EmailCenter;
import com.fr.cluster.ClusterBridge;
import com.fr.compatible.Version;
import com.fr.config.ServerPreferenceConfig;
import com.fr.decision.authority.base.constant.type.operation.ManualOperationType;
import com.fr.decision.authority.base.constant.type.operation.SyncOperationType;
import com.fr.decision.config.AppearanceConfig;
import com.fr.decision.config.DirectoryConfig;
import com.fr.decision.config.FSConfig;
import com.fr.decision.config.UserDataSetConfig;
import com.fr.decision.hyperlink.HyperlinkFactory;
import com.fr.decision.inject.DecisionInjectExtraInfoBuilder;
import com.fr.decision.inject.node.DecisionInjectNode;
import com.fr.decision.inject.node.impl.AbstractDecisionInjectNode;
import com.fr.decision.inject.node.impl.DecisionWebsocketInjectNode;
import com.fr.decision.migration.MigrationContext;
import com.fr.decision.service.DecisionService;
import com.fr.decision.update.acquirer.JSEngineInfo;
import com.fr.decision.webservice.bean.config.ThemeConfigBean;
import com.fr.decision.webservice.utils.theme.ThemeSubConfigFactory;
import com.fr.decision.webservice.v10.config.ConfigService;
import com.fr.decision.webservice.v10.login.LoginService;
import com.fr.decision.webservice.v10.register.RegisterService;
import com.fr.decision.webservice.v10.sms.SMSService;
import com.fr.decision.webservice.v10.system.SystemService;
import com.fr.general.CloudCenter;
import com.fr.general.CloudCenterConfig;
import com.fr.security.SecurityConfig;
import com.fr.security.encryption.irreversible.IrreversibleEncryptors;
import com.fr.security.encryption.transmission.TransmissionEncryptors;
import com.fr.security.encryption.transmission.impl.SM4TransmissionEncryption;
import com.fr.web.WebSocketConfig;

import java.util.*;

public class CustomDecisionSystemInjectNode extends AbstractDecisionInjectNode {

    public String themeId;

    public String getThemeId() {
        return themeId;
    }

    public void setThemeId(String themeId) {
        this.themeId = themeId;
    }

    public CustomDecisionSystemInjectNode(String themeId) {
        this.setThemeId(themeId);
    }

    public String name() {
        return "system";
    }

    @Override
    protected Map<String, Object> calculate(DecisionInjectExtraInfoBuilder var1) throws Exception {
        HashMap var2 = new HashMap();
        if (!var1.isIgnoreDB()) {
            var2.put("adminUser", DecisionService.getInstance().authority().userService().getAdminUserNameList());
            var2.put("manualAuthentication", FSConfig.getInstance().getPassport(ManualOperationType.KEY).markType());
            var2.put("syncAuthentication", FSConfig.getInstance().getPassport(SyncOperationType.KEY).markType());
            var2.put("authConfig", DecisionService.getInstance().authority().authorityService().getAuthorityConfig());
            var2.put("versionInfo", SystemService.getInstance().getSystemVersion());
            var2.putAll(RegisterService.getInstance().getSupportedFunctions());
        }

        var2.put("themeConfig", ConfigService.getInstance().getAllThemes());
        var2.put("styleConfig", ConfigService.getInstance().getStyleConfig());
        var2.put("loginInfoRemind", FSConfig.getInstance().getLoginConfig().getShowLastLoginInfo());
        var2.put("emailAvailable", EmailCenter.isEmailConfigValid());
        var2.put("smsAvailable", SMSService.getInstance().isSMSAvailable());
        var2.put("timeZone", TimeZone.getDefault().getOffset(System.currentTimeMillis()));
        var2.put("hyperlink", HyperlinkFactory.getHyperlinks());
        var2.put("authentication", FSConfig.getInstance().getPassport().markType());
        var2.put("syncDataSet", UserDataSetConfig.getInstance().isTurnOn());
        var2.put("weekBegins", ServerPreferenceConfig.getInstance().getFirstDayOfWeek().getConf());
        var2.put("loginTimeout", FSConfig.getInstance().getLoginConfig().getLoginTimeout());
        var2.put("frontSeed", SecurityConfig.getInstance().getFrontSeed());
        var2.put("frontSM4Key", SM4TransmissionEncryption.getInstance().getTransmissionKey());
        var2.put("transmissionEncryption", TransmissionEncryptors.getInstance().getCurrentTransmissionEncryption().getType());
        var2.put("passwordEncryption", IrreversibleEncryptors.getInstance().getCurrentEncryptionMode().getType());
        var2.put("cookiePath", ServerConfig.getInstance().getCookiePath());
        var2.put("cluster", ClusterBridge.isClusterMode());

        var2.put("syncUserStrategy", UserDataSetConfig.getInstance().getStrategy());
        var2.put("httpOnly", ServerConfig.getInstance().isTokenFromCookie());
        var2.put("runtimeVersion", Version.currVersion().getVersion());
        var2.put("webSocketTokenInHeader", WebSocketConfig.getInstance().isWebSocketTokenInHeader());
        var2.put("jsEngine", JSEngineInfo.getInfoMap());
        var2.put("subThemeConfig", ThemeSubConfigFactory.getConfigById(AppearanceConfig.getInstance().getThemeId()));
        var2.put("sidebarOpen", DirectoryConfig.getInstance().isSidebarOpen());
        var2.put("transferred", MigrationContext.getInstance().isAlreadyTransferred());
        var2.put("urlIP", CloudCenter.getInstance().acquireConf("decision.queryip", ""));
        var2.put("cloudEnabled", CloudCenterConfig.getInstance().isOnline());
        if (AppearanceConfig.getInstance().isCopyrightInfoDisplay()) {
            var2.putAll(LoginService.getInstance().getCopyrightInfo(var1.request()));
        }

        // 设置主题
        if (StringKit.isNotBlank(this.getThemeId())) {
            var2.put("themeId", this.getThemeId());

            // 第三方激活（尝试）
            boolean themeChanged = false;
            List<ThemeConfigBean> themeList = ConfigService.getInstance().getAllThemes();
            for (ThemeConfigBean bean : themeList) {
                // 设置主题
                if (StringKit.equals(bean.getThemeId(), this.getThemeId())) {
                    bean.setActive(true);
                    themeChanged = true;
                } else {
                    bean.setActive(false);
                }
            }
            if (themeChanged) {
                var2.put("themeConfig", themeList);
            }
        }

        return var2;
    }

    public List<DecisionInjectNode> brothers() {
        return Arrays.asList(DecisionWebsocketInjectNode.KEY);
    }
}
